package ${domain}.module.wsys.mpr;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import ${domain}.frame.auth.Token;
import ${domain}.module.wsys.ent.User;
import ${domain}.module.wsys.req.UserFindRequest;

import java.util.List;

/**
 * USER - 用户
 *
 * @author wangbing
 * @since 2020-06-27
 */
@Mapper
public interface UserMapper {

    /**
     * 插入
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 返回数量
     */
    long insert(@Param("request") User request, @Param("token") Token token);

    /**
     * 批量插入
     *
     * @param list  对象集合
     * @param token 令牌
     * @return 返回数量
     */
    long insertBatch(@Param("list") List<User> list, @Param("token") Token token);

    /**
     * 逻辑删除
     *
     * @param id    主键
     * @param token 令牌
     * @return 返回数量
     */
    long delete(@Param("id") Long id, @Param("token") Token token);

    /**
     * 批量逻辑删除
     *
     * @param list  主键集合
     * @param token 令牌
     * @return 返回数量
     */
    long deleteBatch(@Param("list") List<Long> list, @Param("token") Token token);

    /**
     * 更新
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 返回数量
     */
    long update(@Param("request") User request, @Param("token") Token token);

    /**
     * 更新密码
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    long changePwd(@Param("request") User request, @Param("token") Token token);

    /**
     * 查询
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 返回对象
     */
    List<User> find(@Param("request") UserFindRequest request, @Param("token") Token token);

    /**
     * 获得对象
     *
     * @param id    主键
     * @param token 令牌
     * @return 返回对象
     */
    User getById(@Param("id") Long id, @Param("token") Token token);

    /**
     * 获得对象
     *
     * @param list  主键集合
     * @param token 令牌
     * @return 返回对象
     */
    List<User> getByIds(@Param("list") List<Long> list, @Param("token") Token token);
}
